/* This test driver program encodes a data record and writes the encoded 
   result to an output file.. */

#include <stdio.h>
#include <stdlib.h>
#include "NBAP-Constants.h"
#include "NBAP-Containers.h"
#include "NBAP-IEs.h"
#include "NBAP-PDU-Contents.h"
#include "NBAP-PDU-Descriptions.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxFile.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   RL_InformationList_RL_SetupRqstFDD_element rlInfoListElem;
   FDD_DL_CodeInformationItem codeInfoItem;
   RadioLinkSetupRequestFDD_protocolIEs_element protocolIE_1;
   RadioLinkSetupRequestFDD rlSetupRequestFDD;
   InitiatingMessage initMsg;
   NBAP_PDU     pduData;
   OSRTDList    rlInfoList;
   const OSOCTET* msgptr;
   OSOCTET      msgbuf[MAXMSGLEN];
   OSBOOL       verbose = FALSE, trace = TRUE, aligned = TRUE;
   OSCTXT       ctxt;
   const char*  filename = "message.dat";
   int		i, len, stat;

   DECLARE_MEMLEAK_DETECTOR;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   if (rtInitContext (&ctxt) != 0) {
      rtxErrPrint (&ctxt);
      return -1;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, msgbuf, sizeof(msgbuf), aligned);
   pu_setTrace (&ctxt, trace);

   /* Populate RL-InformationList-RL-SetupRqstFDD */

   asn1Init_RL_InformationList_RL_SetupRqstFDD_element (&rlInfoListElem);
   rlInfoListElem.id = ASN1V_id_RL_InformationItem_RL_SetupRqstFDD;
   rlInfoListElem.criticality = Criticality_notify;
   rlInfoListElem.value.rL_ID = 25;               // INTEGER (0..31)
   rlInfoListElem.value.c_ID = 1000;              // INTEGER (0..65535)
   rlInfoListElem.value.firstRLS_indicator = FirstRLS_Indicator_first_RLS;
   rlInfoListElem.value.frameOffset = 99;         // INTEGER (0..255)
   rlInfoListElem.value.chipOffset = 224;         // INTEGER (0..38399)
   rlInfoListElem.value.propagationDelay = 22;    // INTEGER (0..255)
   rlInfoListElem.value.m.diversityControlFieldPresent = 1;
   rlInfoListElem.value.diversityControlField = DiversityControlField_must_not;

   asn1Init_FDD_DL_CodeInformationItem (&codeInfoItem);
   codeInfoItem.dl_ScramblingCode = 0;
   codeInfoItem.fdd_DL_ChannelisationCodeNumber = 255;
   codeInfoItem.transmissionGapPatternSequenceCodeInformation = 
      TransmissionGapPatternSequenceCodeInformation_nocode_change;

   rtxDListInit (&rlInfoListElem.value.dl_CodeInformation);
   rtxDListAppend 
      (&ctxt, &rlInfoListElem.value.dl_CodeInformation, &codeInfoItem);

   /* INTEGER (-350..150) */
   rlInfoListElem.value.initialDL_transmissionPower = -22;
   rlInfoListElem.value.maximumDL_power = 100;
   rlInfoListElem.value.minimumDL_power = -100;
   
   rtxDListInit (&rlInfoList);
   rtxDListAppend (&ctxt, &rlInfoList, &rlInfoListElem);

   /* Populate RadioLinkSetupRequestFDD structure */

   asn1Init_RadioLinkSetupRequestFDD_protocolIEs_element (&protocolIE_1);
   protocolIE_1.id = ASN1V_id_RL_InformationList_RL_SetupRqstFDD;
   protocolIE_1.criticality = Criticality_notify;
   protocolIE_1.value.t = T_NBAPPDUCont_RadioLinkSetupRequestFDD_IEs_id_RL_InformationList_RL_SetupRqstFDD;
   protocolIE_1.value.u._RadioLinkSetupRequestFDD_IEs_id_RL_InformationList_RL_SetupRqstFDD = &rlInfoList;

   asn1Init_RadioLinkSetupRequestFDD (&rlSetupRequestFDD);
   rtxDListAppend (&ctxt, &rlSetupRequestFDD.protocolIEs, &protocolIE_1);

   /* Populate NBAP-PDU structure */

   asn1Init_InitiatingMessage (&initMsg);
   initMsg.procedureID.procedureCode = ASN1V_id_radioLinkSetup;
   initMsg.procedureID.ddMode = ProcedureID_ddMode_fdd;
   initMsg.criticality = Criticality_reject;
   initMsg.messageDiscriminator = MessageDiscriminator_common;
   initMsg.transactionID.t = T_TransactionID_shortTransActionId;
   initMsg.transactionID.u.shortTransActionId = 1;
   initMsg.value.t = T_NBAPPDUDescr_NBAP_EP_radioLinkSetupFDD;
   initMsg.value.u.radioLinkSetupFDD = &rlSetupRequestFDD;

   pduData.t = T_NBAP_PDU_initiatingMessage;
   pduData.u.initiatingMessage = &initMsg;

   /* Encode the entire message */

   if ((stat = asn1PE_NBAP_PDU (&ctxt, &pduData)) == 0) {
      if (trace) {
         printf ("\n******\n");
         printf ("NBAP PDU encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "userInfo");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }

   msgptr = pe_GetMsgPtr (&ctxt, &len);

   /* Write the encoded message out to the output file */

   stat = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != stat) {
      rtxErrPrint (&ctxt);
      return -1;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
